@@ -24,6 +24,7 @@
  *
  */
 
+#include <string.h>
 #include <libbonobo.h>
 #include <glib/gmain.h>
 #include <gnome-speech/gnome-speech.h>
@@ -263,6 +264,50 @@ get_voice_list(void)
 	return(l);
 }
 
+static gboolean
+lang_matches_spec (gchar *lang, gchar *spec)
+{
+	gboolean lang_matches, country_matches = FALSE, encoding_matches = TRUE;
+
+	if (!strcmp (lang, spec))
+	{
+		return TRUE;
+	}
+	else
+	{
+		gchar *lang_country_cp, *spec_country_cp;
+		gchar *lang_encoding_cp, *spec_encoding_cp;
+		lang_matches = !strncmp (lang, spec, 2);
+		if (lang_matches) 
+		{ 
+			lang_country_cp = strchr (lang, '_');
+			spec_country_cp = strchr (spec, '_');
+			lang_encoding_cp = strchr (lang, '@');
+			spec_encoding_cp = strchr (spec, '@');
+
+			if (!lang_country_cp || !spec_country_cp)
+			{
+				country_matches = TRUE;
+			}
+			else
+			{
+				country_matches = !strncmp (lang_country_cp, spec_country_cp, 3);
+			}
+			if (spec_encoding_cp)
+			{
+				if (lang_encoding_cp)
+				{
+					encoding_matches = !strcmp (spec_encoding_cp, lang_encoding_cp); 
+				}
+				else
+				{
+					encoding_matches = FALSE;
+				}
+			}
+		}
+		return lang_matches && country_matches && encoding_matches;
+	}
+}
 
 static void
 voice_list_free(GSList *l)
@@ -276,6 +321,43 @@ voice_list_free(GSList *l)
 	g_slist_free(l);
 }
 
+static GSList *
+prune_voice_list (GSList *l,
+		  const GNOME_Speech_VoiceInfo *info)
+{
+	GSList *cur, *next;
+	GNOME_Speech_VoiceInfo *i;
+
+	cur = l;
+	while (cur) {
+		i = (GNOME_Speech_VoiceInfo *) cur->data;
+		next = cur->next;
+		if (strlen(info->name))
+			if (strcmp (i->name, info->name)) {
+				CORBA_free (i);
+				l = g_slist_remove_link (l, cur);
+				cur = next;
+				continue;
+			}
+		if (strlen(info->language))
+			if (!lang_matches_spec (i->language, info->language)) {
+				CORBA_free (i);
+				l = g_slist_remove_link (l, cur);
+				cur = next;
+				continue;
+			}
+		if ((info->gender == GNOME_Speech_gender_male || info->gender == GNOME_Speech_gender_female) &&
+		    (info->gender != i->gender)) {
+			CORBA_free (i);
+			l = g_slist_remove_link (l, cur);
+			cur = next;
+			continue;
+		}
+		cur = next;
+	}
+	return l;
+}
+
 
 static GNOME_Speech_VoiceInfoList *
 voice_info_list_from_voice_list(GSList *l)
@@ -352,6 +434,7 @@ dectalk_getVoices(PortableServer_Servant servant,
 	GSList *l;
 
 	l = get_voice_list();
+	l = prune_voice_list (l, voice_spec);
 	rv = voice_info_list_from_voice_list(l);
 	voice_list_free(l);
 
@@ -380,7 +463,17 @@ dectalk_createSpeaker(PortableServer_Servant servant,
                       CORBA_Environment *ev)
 {
 	DectalkSynthesisDriver *d = dectalk_synthesis_driver_from_servant (servant);
-	DectalkSpeaker *s = dectalk_speaker_new(G_OBJECT(d), voice_spec);
+	DectalkSpeaker *s;
+	GSList *l;
+	GNOME_Speech_VoiceInfo *info;
+
+	l = get_voice_list ();
+	l = prune_voice_list (l, voice_spec);
+	if (l)
+		info = l->data;
+	else
+		info = NULL;
+	s = dectalk_speaker_new(G_OBJECT(d), info);
 
 	return(CORBA_Object_duplicate(
 		       bonobo_object_corba_objref(BONOBO_OBJECT(s)), ev));
